(*****************************************************************************)
(*  robot_concurrent.ml :                                                    *)
(*                                                                           *)
(* POD : E.Chailloux  P.Manoury  B.Pagano                                    *)
(*****************************************************************************)

(*************************************************************************)
(*                       ROBOT ET MONDE ACTIFS                           *)
(*************************************************************************)

open Robot_def
open Robot_display 
open Robot_actif


class virtual ['a] mondeV_actifV2 = 
  object
    inherit ['a] mondeV_actifV 
end


class virtual ['a] mondeV_concur () =
  object (s)
    inherit ['a] mondeV_actifV2 as super 
    val sem = Mutex.create ()

    initializer Mutex.lock sem

    method private move_robot r =
      let pos = s#normalize (r#next_position s)
      in 
	Mutex.lock sem ;
	(if (s#is_free pos && s#is_legal pos) then r#set_pos pos ) ;
        Mutex.unlock sem;

    method enter r = 
      Thread.create (fun () -> while true do s#move_robot r done) () ; ()


    method go = Mutex.unlock sem
    method stop = Mutex.try_lock sem ; ()
end






class ['a] ferme_concur_graph hi li gc =
 object (s)

    inherit ['a] mondeV_concur () as super_conc
    inherit ['a] mondeV_graph gc
    inherit ['a] monde_ferme hi li as super_fer

    method private move_robot r = 
      super_conc#move_robot r ; 
      r#display ;
      Thread.delay ((Random.float 0.3) +. 0.2)

    method enter r = super_fer#enter r ; super_conc#enter r
  end 


